/*************************************************************************
  P I C K   L I S T    C L A S S
**************************************************************************
 The purpose of this class is to encapsulate strings used for displays
 suck as radio buttons, check boxes and drop-down selections and which
 are associated with numerical constants.  For example OFFICE_LON=4="London"
 
 The data structure is basically an array of strings keyed to constants
 although straight sequential indexing can be used without explicit
 constants.
 
 The main functions are:
  Convert both ways between index number and string
  Create HTML for select, input-radio and input-checkbox

 Notes
  The terminology used is 'string' for an item string and 'constant' for index to it

*************************************************************************/


// ------------------------------------------------------------------------------------
function PICKLIST(StringsArray,IndicesArray){
// Create list
// - StringsArray is required being the strings to use for displays
// - IndicesArray is optional containing constants.  NB Numeric values only
// Examples
//   p = new PICKLIST(array('Up','Down','Left','Right'));
//   p = new PICKLIST(array('London','Paris','NEW York'),
//                    array(LOC_LON,LOC_PAR,LOC_NY));  // 'Constants' defined elsewhere
// ------------------------------------------------------------------------------------
  if(StringsArray.length<1){
    throw new Error('PICKLIST requires a populated string array');  
  }  
  for(var i = 0;i<StringsArray.length;i++){
    var sai = StringsArray[i];
    if(sai===''){
      throw new Error('PICKLIST doesn\'t allow empty strings');   
    }  
    if(typeof sai != 'string'){
      throw new Error('PICKLIST doesn\'t allow non-strings');    
    }  
    for(var j=i+1;j<StringsArray.length;j++){
      if(sai===StringsArray[j]){
        throw new Error('PICKLIST doesn\'t allow duplicate strings');    
      }  
    }    
  }  
  this.strings = StringsArray;
  this.index = [];
  // So far the simple case without special numbers.  
  // We may have special constants in which case
  // there is more work to be done  
  if(arguments.length==2){
    if(IndicesArray.length!=StringsArray.length){
      throw new Error('PICKLIST requires arrays of the same length');    
    }  
    // fill the index but check for duplicates
    // make sure these are numbers
    for(var i = 0;i<IndicesArray.length;i++){
      var con = IndicesArray[i];
      if(typeof IndicesArray[i] != 'number'){
        throw new Error('PICKLIST constants must be numbers.  Index of bad item is '+i.toString());  
      }
      if(isNaN(con)||(! isFinite(con))){
        throw new Error('PICKLIST constants must be valid numbers.  Index of bad item is '+i.toString());  
      }  
      var coni = Math.floor(con);
      if(coni != con){
        throw new Error('PICKLIST constants must be integers.  Index of bad item is '+i.toString()); 
      }  
      // look for dups already collected
      for(var j=0;j<this.index.length;j++){
        if(coni===this.index[j]){
          throw new Error('PICKLIST doesn\'t allow duplicate constants   Index of bad item is '+i.toString());   
        }  
      }  
      this.index[i]=coni;
    }
  }


// ------------------------------------------------------------------------------------
  this.GetItemString = function(Index) {
// Return the string for specified index  
// ------------------------------------------------------------------------------------
    var i = Number(Index);
    if(isNaN(i)==true){throw new Error('PICKLIST.GetString() needs numeric argument');} 
    // we may need to do reverse lookup using this.index
    if(this.IsIndexed()==true){
      i = this.FindConstant(i);
      if(i<0){throw new Error('PICKLIST.GetString() has unknown value for argument');}  
    }
    // sanity check range 
    if((i<0)||(i>this.strings.length-1)){throw new Error('PICKLIST.GetString() argument out of range');}  
    return this.strings[i];
  }

// ------------------------------------------------------------------------------------
  this.GetItemConstant = function(Str) {
// Return the constant for specified string 
// NOTE This will throw an error if not found
// Case sensitive
// ------------------------------------------------------------------------------------
    var found = -1;
    for(var i=0;i<this.strings.length;i++){
      if(this.strings[i]==Str){
        found=i;
        break;
      }
    }
    if(found == -1){
      throw new Error('PICKLIST.GetItemConstant('+Str+") couldn't find string");   
    }
    if(this.IsIndexed()==true){
      found = this.index[found];
    }
    return found;
  }    
  
// ------------------------------------------------------------------------------------
  this.IsIndexed = function(){return (this.index.length>0);}
// PRIVATE
// ------------------------------------------------------------------------------------
  

// ------------------------------------------------------------------------------------
  this.FindConstant = function(Constant){                 
//  PRIVATE Looks to see where in the index the specifed constant appears   
//  If not found then return -1
//  ------------------------------------------------------------------------------------
    var rv = -1;   // default = not found
    for(var i=0;i<this.index.length;i++){
      if(Constant==this.index[i]){
        rv = i;
        break;
      }
    } 
    return rv;    
  }
  
  
// ------------------------------------------------------------------------------------
  this.toString  = function() {  
// DEBUGGING  
// ------------------------------------------------------------------------------------
    var s = '';
    var linear = (this.index.length==0);
    for(var i=0;i<this.strings.length;i++){
      var con = i;
      if(linear==false){con = this.index[i];}
      s += '['+con.toString()+']="'+this.strings[i]+'" ';
    }  
    return s; 
  }    

// ------------------------------------------------------------------------------------
  this.IsInSelected = function (Str,Ix,Selected){
// PRIVATE
// Return true of string or index match selected
// Selected can be Null, String, Constant(number) or array of 0 or many strings or numbers
// ------------------------------------------------------------------------------------
    switch (typeof Selected){
    case 'undefined',null   : return false; break;
    case 'string'           : return (Str==Selected); break;
    case 'number'           : return (Ix==Selected); break;
    case 'object' :     // array
      var rv = false;
      for(var i=0;i<Selected.length;i++){
        if(this.IsInSelected(Str,Ix,Selected[i])){
          rv = true;
          break;
        }
      }
      return rv;
    default : return false;
    }    
  }

// ------------------------------------------------------------------------------------
  this.OptionsHtml = function(Selected){
//  Return the <options> for a <select>
//  Selected can be Null, String, Constant(number) or array of 0 or many strings or numbers
// ------------------------------------------------------------------------------------
    var html = '';
    var linear = (this.index.length==0);
    for(var i=0;i<this.strings.length;i++){
      var s = this.strings[i]; 
      var ix = (linear==false) ? this.index[i] : i;
      var sel = (this.IsInSelected(s,ix,Selected)) ? ' selected ' : '';
      html += '<option'+sel+'>'+s+'</option>';
    }
    return html; 
  }  

// ------------------------------------------------------------------------------------
  this.RadioHtml= function(Selected,Name,Markup){
//  Return a set of radio buttons
//  Selected can be Null, String, Constant(number) or array of 0 or many strings or numbers
//  Name identifies the set
//  Markup is added between (not at end) buttons
// ------------------------------------------------------------------------------------
    var html = '';
    var linear = (this.index.length==0);
    for(var i=0;i<this.strings.length;i++){
      var s = this.strings[i]; 
      var ix = (linear==false) ? this.index[i] : i;
      var chk = (this.IsInSelected(s,ix,Selected)) ? ' checked ' : '';
      html += s+'<input type=radio name="'+Name+'" '+chk+'>';
      if(i<(this.strings.length-1)){ html += Markup;}
    }
    return html; 
  }  

// ------------------------------------------------------------------------------------
  this.ChecksHtml= function(Selected,Name,Markup){
//  Return a set of radio buttons
//  Selected can be Null, String, Constant(number) or array of 0 or many strings or numbers
//  Name identifies the set
//  Markup is added between (not at end) buttons
// ------------------------------------------------------------------------------------
    var html = '';
    var linear = (this.index.length==0);
    for(var i=0;i<this.strings.length;i++){
      var s = this.strings[i]; 
      var ix = (linear==false) ? this.index[i] : i;
      var chk = (this.IsInSelected(s,i,Selected)) ? ' checked ' : '';
      html += s+'<input type=checkbox name="'+Name+'" '+chk+'>';
      if(i<(this.strings.length-1)){ html += Markup;}
    }
    return html; 
  }  

}
